//
//	Copyright (c) 2005 Nemetschek North America 
//	All Rights Reserved.
//
// Created by Vladislav Stanev

#include "StdHeaders.h"

#include "VWFC/PluginSupport/VWExtensions.h"
#include "VWFC/PluginSupport/VWExtensionMenu.h"

using namespace VWFC::PluginSupport;


// --------------------------------------------------------------------------------------------------------
VWExtensionVSFunctions::VWExtensionVSFunctions(	CallBackPtr cbp,
												const SFunctionDef* arrFunctions)
	: fCBP( cbp )
	, farrFunctions( arrFunctions )
	, fLegacyEventSink( NULL )
	, fHasProtection( false )
	, fProtectionSink( NULL )
{
	fHasProtection	= this->GetProtectionSinkIID( fProtectionSinkIID );
}

VWExtensionVSFunctions::~VWExtensionVSFunctions()
{
}

void VWExtensionVSFunctions::OnRefCountZero()
{
	if ( fLegacyEventSink ) {
		delete fLegacyEventSink;
		fLegacyEventSink	= NULL;
	}

	if ( fProtectionSink ) {
		delete fProtectionSink;
		fProtectionSink	= NULL;
	}

	// unload resources
	::GS_VWUnloadInterfaceResource( this );
}

IEventSink* VWExtensionVSFunctions::QueryEventSink(const TSinkIID& iid)
{
	IEventSink*	pResult	= NULL;
	if ( IID_LegacyEventSink == iid ) {
		if ( fLegacyEventSink == NULL ) {
			fLegacyEventSink	= this->CreateLegacyEventSink( this );
			if ( fLegacyEventSink ) {
				this->InitRoutines();
				fLegacyEventSink->AddRef();
				VWVSFunctions_EventSink*	vwfcSink	= dynamic_cast<VWVSFunctions_EventSink*>( fLegacyEventSink );
				if ( vwfcSink ) {
					vwfcSink->SetUniversalName( this->GetUniversalName() );
				}
			}
		}
		pResult	= fLegacyEventSink;
	}
	else if ( fHasProtection && fProtectionSinkIID == iid ) {
		if ( fProtectionSink == NULL ) {
			fProtectionSink	= this->CreateProtectionSink( this );
			if ( fProtectionSink == NULL ) {
				fHasProtection	= false;
			}
		}

		if ( fProtectionSink ) {
			fProtectionSink->AddRef();
			pResult	= fProtectionSink;
		}
	}

	return pResult;
}

bool VWExtensionVSFunctions::GetProtectionSinkIID(TSinkIID& outIID)
{
	return false;
}

IProviderProtection* VWExtensionVSFunctions::CreateProtectionSink(IVWUnknown* parent)
{
	return NULL;
}

Uint32 VWExtensionVSFunctions::Initialize()
{
	return kExtensionVSFunctionsInitFlag_None;
}

size_t VWExtensionVSFunctions::GetFunctionsCount()
{
	size_t				functionsCount	= 0;
	const SFunctionDef*	currFunc		= farrFunctions;
	while ( currFunc && currFunc->fName ) {
		functionsCount++;
		currFunc++;
	}

	return functionsCount;
}

const TXString& VWExtensionVSFunctions::GetFunctionName(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&		funcDef	= farrFunctions[ funcIndex ];
	TStringCache::iterator	it		= fmapStringCache.find( funcDef.fName );
	if ( it == fmapStringCache.end() ) {
		TResultPair	res	= fmapStringCache.insert( TStringCache::value_type( funcDef.fName, funcDef.fName ) );
		it	= res.first;
	}
	return it->second;
}

const TXString& VWExtensionVSFunctions::GetFunctionCategory(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&		funcDef	= farrFunctions[ funcIndex ];
	TStringCache::iterator	it		= fmapStringCache.find( funcDef.fCategory );
	if ( it == fmapStringCache.end() ) {
		TResultPair	res	= fmapStringCache.insert( TStringCache::value_type( funcDef.fCategory, funcDef.fCategory ) );
		it	= res.first;
	}
	return it->second;
}

const TXString& VWExtensionVSFunctions::GetFunctionDescription(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&		funcDef	= farrFunctions[ funcIndex ];
	TStringCache::iterator	it		= fmapStringCache.find( funcDef.fDescription );
	if ( it == fmapStringCache.end() ) {
		TResultPair	res	= fmapStringCache.insert( TStringCache::value_type( funcDef.fDescription, funcDef.fDescription ) );
		it	= res.first;
	}
	return it->second;
}

Sint32 VWExtensionVSFunctions::GetFunctionVersion(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&	funcDef	= farrFunctions[ funcIndex ];
	return funcDef.fVersion;
}

Sint8 VWExtensionVSFunctions::GetFunctionScope(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&	funcDef	= farrFunctions[ funcIndex ];
	return funcDef.fScope;
}

bool VWExtensionVSFunctions::GetFunctionHasReturnValue(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&	funcDef	= farrFunctions[ funcIndex ];
	return funcDef.fHasReturnValue;
}

size_t VWExtensionVSFunctions::GetFunctionParamsCount(size_t funcIndex)
{
	VWFC_ASSERT( funcIndex < this->GetFunctionsCount() );
	const SFunctionDef&	funcDef	= farrFunctions[ funcIndex ];

	size_t						paramsCount		= 0;
	const SFunctionParamDef*	currParam		= funcDef.fParams;
	while ( currParam && currParam->fName ) {
		paramsCount++;
		currParam++;
	}

	return paramsCount;
}

const TXString& VWExtensionVSFunctions::GetFunctionParamName(size_t funcIndex, size_t paramIndex)
{
	VWFC_ASSERT( paramIndex < this->GetFunctionParamsCount( funcIndex ) );
	const SFunctionDef&			funcDef		= farrFunctions[ funcIndex ];
	const SFunctionParamDef&	paramDef	= funcDef.fParams[ paramIndex ];
	TStringCache::iterator		it			= fmapStringCache.find( paramDef.fName );
	if ( it == fmapStringCache.end() ) {
		TResultPair	res	= fmapStringCache.insert( TStringCache::value_type( paramDef.fName, paramDef.fName ) );
		it	= res.first;
	}
	return it->second;
}

EPluginLibraryArgType VWExtensionVSFunctions::GetFunctionParamType(size_t funcIndex, size_t paramIndex)
{
	VWFC_ASSERT( paramIndex < this->GetFunctionParamsCount( funcIndex ) );
	const SFunctionDef&			funcDef		= farrFunctions[ funcIndex ];
	const SFunctionParamDef&	paramDef	= funcDef.fParams[ paramIndex ];
	return paramDef.fType;
}

// --------------------------------------------------------------------------------------------------------
// predefine these hidden memory managment functions
namespace VWFC
{
	void	AllocateMemoryManagers(VWFC::MemoryManager::MManager** ppOutOldManager, VWFC::MemoryManager::MManager** ppOutOldVWManager);
	void	FreeMemoryManagers(VWFC::MemoryManager::MManager* pOldManager, VWFC::MemoryManager::MManager* pOldVWManager);
}

// --------------------------------------------------------------------------------------------------------
VWVSFunctions_EventSink::VWVSFunctions_EventSink(IVWUnknown* parent)
	: VCOMImpl<ILegacyEventSink>( parent )
	, fRoutines( NULL )
{
	
}

VWVSFunctions_EventSink::~VWVSFunctions_EventSink()
{
	SRoutinesDef*	currRoutine	= fRoutines;
	while ( currRoutine ) {
		VWPluginLibraryRoutine*	routine	= currRoutine->fRoutine;
		SRoutinesDef*			prev	= currRoutine->fPrev;

		if ( ! currRoutine->fIsLocalMemory && routine ) {
			delete routine;
		}
		delete currRoutine;

		currRoutine	= prev;
	}
	fRoutines = NULL;
}

void VWVSFunctions_EventSink::SetUniversalName(const TXString& name)
{
	fUnivName	= name;
}

void VWVSFunctions_EventSink::AddRoutine(VWPluginLibraryRoutine* routine, bool isLocalMemory)
{
	SRoutinesDef*	newRoutineDef = new SRoutinesDef;

	newRoutineDef->fRoutine			= routine;
	newRoutineDef->fIsLocalMemory	= isLocalMemory;
	newRoutineDef->fPrev			= fRoutines;

	fRoutines					= newRoutineDef;
}

long VWVSFunctions_EventSink::LegacyMain(long action, long message1, long )
{
	PluginLibraryArgTable*		pArgTable = static_cast<PluginLibraryArgTable*>( (void*) message1 );
	VWPluginLibraryArgTable		argTable( pArgTable );

	long reply	= 0;

	// create the memory namagers
	// and verify their presents
	VWFC::MemoryManager::MManager* pOldManager		= NULL;
	VWFC::MemoryManager::MManager* pOldVWManager	= NULL;
	VWFC::AllocateMemoryManagers( & pOldManager, & pOldVWManager );

	VWFC_VERIFY( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL );
	if ( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL ) {

		SRoutinesDef*	currRoutine	= fRoutines;
		while ( currRoutine ) {
			VWPluginLibraryRoutine*	routine	= currRoutine->fRoutine;
			if ( routine ) {
				routine->DispatchRoutine( action, argTable );
			}

			currRoutine	= currRoutine->fPrev;
		}

		// dump local memory leaks
		if ( VWFC::GetMManager() ) {
			VWFC::GetMManager()->DumpMemory( fUnivName );
		}
	}

	// release the memory managers
	VWFC::FreeMemoryManagers( pOldManager, pOldVWManager );

	return reply;
}
